/*******************************************************************************
  File : book.c

  Description : Gestion de la bibliothque des ouvertures.
*******************************************************************************/

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des diffrents modules du programme : */
#include "book.h"
#include "debug.h"
#include "hash.h"
#include "moves.h"
#include "tools.h"

/*******************************************************************************
  Constants :
*******************************************************************************/

/* Nom des fichiers utiliss par Sissa : */
#define OPENINGS_FILE        "etc/opening_book.txt"

/* Valeurs limites pour quelques constantes gnrales : */
#define OPENING_LINES_LIMIT  1024
#define OPENING_LINE_LENGTH   256

/*******************************************************************************
  Data :
*******************************************************************************/

/* Fichier contenant les ouvertures : */
FILE *openings_file;

/*******************************************************************************
  External functions :
*******************************************************************************/

/*******************************************************************************
  Function     : select_move
  Description  : Slection d'un coup dans la bibliothque d'ouvertures.
  Parameters   : out - coup slectionn dans la bibliothque.
  Return value : Le programme a trouv un coup (true) ou non (false).
  Validation   : 1
*******************************************************************************/
bool select_move (move_s *move) {

/* ------------------------------- Local data ------------------------------- */

bool book_move = false; /* Indique si le coup d'une ouverture est dj jou. */
bool line_to_keep;      /* Indique si une ouverture se rapporte  la partie. */
bool move_in_line;      /* Indique si un coup est dans une ligne de jeu. */

char lan_move[6];                          /* Coup en notation algbrique. */
char lan_played_moves[BOOK_PLY_LIMIT][6];  /* Liste des coups jous et convertis
                                              en notation algbrique. */
char moves_line[STRING_LENGTH];     /* Liste des coups d'une ligne de jeu. */
char opening_line[STRING_LENGTH];   /* Ligne de jeu issue de la bibliothque. */
char opening_lines[OPENING_LINES_LIMIT]    /* Liste des ouvertures qui */
                  [BOOK_PLY_LIMIT * 6];    /* se rapportent  la partie. */
char *pointer;                      /* Pour parcourir la liste des coups. */

clock_t start_chronometer;   /* Pour calculer le temps mis  chercher */
clock_t stop_chronometer;    /* un coup dans la bibliothque. */

int i, j;
int count_lines;        /* Nombre d'ouvertures trouves dans la bibliothque. */
int selected_line;      /* Numro de l'ouverture slectionne. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("select_move\n", true, &select_move_calls);
#endif /* DEBUG */

  /* Mmorise le dbut de la recherche. */
  start_chronometer = clock ();

  /* Initialisation,  partir de l'heure actuelle, d'une nouvelle squence
     de nombres pseudo-alatoires pour varier le choix des ouvertures. */
  srand(time(NULL));

  /* Si le fichier qui contient les ouvertures n'est pas trouv, */
  if ((openings_file = fopen ((active_color ? OPENINGS_FILE : \
    OPENINGS_FILE), "r")) == NULL) {

    /* alors un message est affich et la recherche est interrompue. */
    if (uci_mode) {

      tools.send_output (true,
        "info string %s can't open the openings file %s",
        PRODUCT_NAME, OPENINGS_FILE);
    }
    else {

      tools.send_output (true,
        "\n   WARNING : %s can't open the openings file %s,\n"
        "             however it will continue to play.\n\n",
        PRODUCT_NAME, (active_color ? OPENINGS_FILE : OPENINGS_FILE));
    }
  }
  else {

    /* Premire tape : construction de la liste des coups jous.
       ---------------------------------------------------------- */

    /* Tous les coups jous depuis le dbut de la partie, */
    for (i = 0; i < game_ply; i++) {

      /* sont rangs dans un tableau en notation algbrique. */
      tools.convert_pgm_move (&played_moves[i], lan_played_moves[i]);
    }

#ifdef DEBUG
debug.print("game_ply %d - played moves",game_ply);
while (--i >= 0) {
  debug.print(" .%s.",lan_played_moves[i]);
}
debug.print("\n");
#endif /* DEBUG */

    /* Seconde tape : recherche des lignes d'ouvertures possibles.
       ------------------------------------------------------------ */

    /* Le programme va rechercher celles qui contiennent tous les coups jous.
       cette faon de procder va permettre de tenir compte des inversions de
       coups. Le nombre des lignes slectionnes sera limit pour viter les
       dbordements. */
    count_lines = 0;
    while (!feof (openings_file) && count_lines < OPENING_LINES_LIMIT) {

      /* Lecture d'une ligne d'ouverture dans le fichier des ouvertures. */
      fgets (opening_line, STRING_LENGTH, openings_file);

      /* Si la ligne n'est pas un commentaire, */
      if (opening_line[0] != '#') {

        /* alors le caractre de retour  la ligne est supprim, */
        opening_line[strlen(opening_line) - 1] = '\0';
#ifdef DEBUG
debug.print ("\nbook line (size %d) .%s.\n", strlen (opening_line), opening_line);
#endif /* DEBUG */

        line_to_keep = true;
        i = game_ply;

        /* et tant que la ligne d'ouverture est semblable, pour chaque coup
           jou depuis le dbut de la partie, */
        while (--i >= 0 && line_to_keep) {

          move_in_line = false;
          strcpy (moves_line, opening_line);
          j = 0;

          /* le programme va vrifier qu'il est prsent dans la ligne
             d'ouverture, */
          for (pointer = strtok (moves_line, " ");
               pointer != NULL && j <= game_ply && !move_in_line;
               pointer = strtok (NULL, " ")) {

#ifdef DEBUG
debug.print ("  compare played move .%s. and book move .%s.\n",
             lan_played_moves[i],pointer);
#endif /* DEBUG */
            if (!strcmp (lan_played_moves[i], pointer)) {

#ifdef DEBUG
debug.print ("  played move .%s. is found in %d\n", pointer, j+1);
#endif /* DEBUG */
              move_in_line = true;
            }

            j++;
          }

          /* Si le coup n'est pas prsent dans la ligne d'ouverture, */
          if (!move_in_line) {

#ifdef DEBUG
debug.print ("  played move .%s. is not found \n", lan_played_moves[i]);
#endif /* DEBUG */
            /* alors cette dernire ne sera pas conserve. */
            line_to_keep = false;
          }
        }

        /* Lorsque le programme a vrifi que tous les coups jous sont prsents
           dans la ligne d'ouverture lue dans le fichier, */
        if (line_to_keep) {

#ifdef DEBUG
debug.print ("  line is correct\n");
#endif /* DEBUG */
          /* celle-ci sera mmorise pour l'tape suivante. */
          strcpy (opening_lines[count_lines++], opening_line);
        }
      }
    }
#ifdef DEBUG
debug.print("\n");
#endif /* DEBUG */

    /* Troisime tape : Slection d'une ligne d'ouverture.
       ---------------------------------------------------- */

    /* Si des ouvertures sont trouves, */
    if (count_lines) {

      /* alors l'une d'entre elles sera slectionne alatoirement. */
      selected_line = rand() % count_lines;
#ifdef DEBUG
debug.print("\n");
#endif /* DEBUG */

      if (uci_mode) {

        tools.send_output (true, "info string Openings book line %d/%d\n",
          selected_line + 1, count_lines);
      }
      else {

        tools.send_output (true, "\n   Openings book line (%d/%d)\n",
          selected_line + 1, count_lines);
      }

#ifdef DEBUG
debug.print ("selected_line (%d/%d) .%s.\n\n",
             selected_line+1, count_lines, opening_lines[selected_line]);
#endif /* DEBUG */

      /* Quatrime tape : Slection du prochain coup  jouer.
         ----------------------------------------------------- */

      strcpy (moves_line, opening_lines[selected_line]);
      j = 0;

      /* Pour chaque coup de la ligne de jeu slectionne, */
      for (pointer = strtok (moves_line, " ");
           pointer != NULL && j <= game_ply && !book_move;
           pointer = strtok (NULL, " ")) {

        i = game_ply;
        move_in_line = false;

        /* le programme parcourera toute la liste des coups jous, tant qu'il
           ne sera pas trouv. */
        while (--i >= 0 && !move_in_line) {
#ifdef DEBUG
debug.print ("  compare book move .%s. and played move .%s.\n",
             lan_played_moves[i],pointer);
#endif /* DEBUG */

          if (!strcmp (lan_played_moves[i], pointer)) {

            move_in_line = true;
          }
        }

        j++;

        /* Si le coup n'est toujours pas trouv une fois la liste parcourue, */
        if (!move_in_line) {

          /* alors il est celui qui doit tre jou par le programme. */
          book_move = true;
          strcpy (lan_move, pointer);
#ifdef DEBUG
debug.print ("  move to play .%s. in %d\n", pointer, j);
#endif /* DEBUG */
        }
      }

      /* Une dernire vrification est faite pour s'assurer que le coup est
         jouable dans la position courante. */
      if (tools.verify_move (lan_move)) {

        /* Enfin le coup trouv est converti et renvoy  l'appelant, */
        tools.convert_lan_move (lan_move, move);

        /* La fin de la recherche est dclenche, */
        stop_chronometer = clock ();

        /* et le rsultat est affich. */
        if (uci_mode) {

          tools.send_output (true,
            "info string Elapsed time to select book move : %.1f s\n",
            (stop_chronometer - start_chronometer) / (double) CLOCKS_PER_SEC);
        }
        else {

          tools.send_output (true,
            "   Elapsed time to select book move : %.1f s\n",
            (stop_chronometer - start_chronometer) / (double) CLOCKS_PER_SEC);
        }
      }
      else {

        /* sinon le coup n'est pas jouable, */
        book_move = false;

        /* ce qui signifie que la ligne de jeu est probablement errone. */
        if (uci_mode) {

          tools.send_output (true,
            "info string WARNING : Opening line is probably bad : %s",
            opening_lines[selected_line]);
        }
        else {

          tools.send_output (true,
            "\n   WARNING : Opening line is probably bad :\n"
            "             %s\n", opening_lines[selected_line]);
        }
      }
    }

    /* Le fichier des ouvertures est ferm, */
    fclose (openings_file);
  }

  /* et le rsultat de la recherche est retourn. */
  return book_move;
}
/* End of function : select_move */

/*******************************************************************************
  Function     : make_book
  Description  : Permet de construire le fichier des lignes d'ouvertures 
                 partir d'un fichier de parties au format texte.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void make_book (void) {

/* ------------------------------- Local data ------------------------------- */

FILE *games_file;                 /* Fichier contenant la liste des parties. */

char game[STRING_LENGTH];         /* Liste des coups de la partie. */
char lan_move[6];                 /* Coup en notation algbrique. */

int book_ply;                     /* Nombre de coup de la partie. */
int count_lines = 0;              /* Nombre d'ouvertures qui seront enregistres
                                     dans la bibliothque. */
int i, j;                         /* Indices de boucle. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("make_book\n", true, &make_book_calls);
#endif /* DEBUG */

  /* Si le programme ne trouve pas la liste des parties, */
  if (   (games_file = fopen (GAMES_FILE, "r")) == NULL
      || (openings_file = fopen (OPENINGS_FILE, "w")) == NULL) {

    /* alors l'information est affiche et la construction est interrompue. */
    tools.send_output (true,
      "\n   ERROR : %s can't open the games and openings file\n"
      "           (%s et/ou %s).\n\n", PRODUCT_NAME, GAMES_FILE, OPENINGS_FILE);
  }
  else {

    /* Tant que le fichier n'est pas entirement lu. */
    while (!feof (games_file)) {

      count_lines++;

      /* Les 4 premires lignes correspondent  l'en-tte d'une partie et ne
         sont pas conserves : */
      for (i = 0; i < 5; i++) {

        fgets (game, STRING_LENGTH, games_file);
      }

      book_ply = 0;
      i = 0;

      /* La lecture de la partie se fera jusqu' sa fin ou au nombre de coups
         maximum autoriss pour une ligne de jeu dans la bibliothque. */
      while (i < strlen (game) && book_ply < BOOK_PLY_LIMIT) {

        /* Saut du numro du coup. */
        while ((game[i++]) != (int) '.');

        lan_move[4] = '\0';
        lan_move[5] = '\0';

        /* Lecture du coup des blancs. */
        j = 0;
        while (game[i] != (int) ' ') {
          if (   (game[i] >= '0' && game[i] <= '8')
              || (game[i] >= 'a' && game[i] <= 'r')) {
            lan_move[j++] = game[i];
          }
          i++;
        }

        /* Conversion du roque blanc. */
        if (!strncmp (lan_move, "000", 3)) {
          strcpy (lan_move, "e1c1");
        }
        else if (!strncmp (lan_move, "00", 2)) {
          strcpy (lan_move, "e1g1");
        }

        /* Ecriture du coup des blancs. */
        book_ply++;
        fprintf (openings_file, lan_move);
        fprintf (openings_file, " ");

        if (++i < strlen (game)) {

          lan_move[4] = '\0';
          lan_move[5] = '\0';

          /* Lecture du coup des noirs. */
          j = 0;
          while (game[i] != (int) ' ') {
            if (   (game[i] >= '0' && game[i] <= '8')
                || (game[i] >= 'a' && game[i] <= 'r')) {
              lan_move[j++] = game[i];
            }
            i++;
          }

          /* Conversion du roque noir. */
          if (!strncmp (lan_move, "000", 3)) {
            strcpy (lan_move, "e8c8");
          }
          else if (!strncmp (lan_move, "00", 2)) {
            strcpy (lan_move, "e8g8");
          }

          /* Ecriture du coup des noirs. */
          book_ply++;
          fprintf (openings_file, lan_move);
          fprintf (openings_file, " ");
        }
      }

      /* La ligne de jeu est termine. */
      fprintf (openings_file, "\n");
    }

    fclose (games_file);
    fclose (openings_file);

    tools.send_output (true, "   %s write %d lines in its openings book.\n\n",
      PRODUCT_NAME, count_lines, OPENINGS_FILE);
  }
}
/* End of function : make_book */

/*******************************************************************************
  Module declaration :
*******************************************************************************/
book_module book = {

  select_move,
  make_book
};
