/*******************************************************************************
  File : data.c

  Description : Donnes globales du programme avec leur description.
*******************************************************************************/

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des donnes gnrales du programme : */
#include "main.h"

/* Fichiers en-tte des donnes globales du programme : */
#include "data.h"

/*******************************************************************************
  Data :
*******************************************************************************/

/* Fichier journal pour enregistrer les changes avec le programme : */
FILE *log_file;

/* Indique si la bibliothque des ouvertures doit tre utilise : */
bool book_use;

/* Indique si le programme joue automatiquement  la rception d'un coup :
   (In UCI mode, the engine will always be in forced mode which means
    it should never start calculating or pondering without receiving a
    "go" command first) : */
bool forced_mode;

/* Indique si le fichier journal est utilis. */
bool log_use;

/* Indique si l'chiquier doit tre affich en mode console : */
bool show_board;

/* The program stop its search when the maximum depth is reached. */
bool search_on_depth_limit;

/* Indique si le programme s'excute en mode test : */
bool test_mode;

/* Indique si les changes entre le programme et l'extrieur se font en
   respectant le protocole UCI : */
bool uci_mode;

/* La position de dpart standard au format FEN/EPD : */
char start_position[STRING_LENGTH] =
  { "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1" };

/* Permet le temps depuis le dbut de la rflexion du programme : */
clock_t start_thinking_time;

/* Masques pour isoler petit et grand roque de chaque couleur :
  (voir la description de castle_status) */
const int castle_by_color_mask[2] = { 0x3C, 0x33 };

/* Masques pour isoler chaque donne de la variable castle_status :
  (voir la description de castle_status) */
const int castle_data_mask[7] = { 0, 0x1, 0x2, 0x4, 0x8, 0x10, 0x20 };

/* Masques pour isoler les donnes de la variable castle_status, afin de savoir
   si les roques noirs et blancs sont effectus :
  (voir la description de castle_status) */
const int castle_done_mask[2]     = { 0x20, 0x10 }; /* Roques effectus */

/* Masques pour isoler les donnes de la variable castle_status, afin de savoir
   si les roques noirs et blancs sont possibles :
  (voir la description de castle_status) */
const int castle_possible_mask[2] = { 0x0C, 0x03 }; /* Roques possibles */

/* Pour chaque case de l'chiquier, donne le numro de la colonne : */
const int file[120] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 1, 2, 3, 4, 5, 6, 7, 8, 0,
 0, 1, 2, 3, 4, 5, 6, 7, 8, 0,
 0, 1, 2, 3, 4, 5, 6, 7, 8, 0,
 0, 1, 2, 3, 4, 5, 6, 7, 8, 0,
 0, 1, 2, 3, 4, 5, 6, 7, 8, 0,
 0, 1, 2, 3, 4, 5, 6, 7, 8, 0,
 0, 1, 2, 3, 4, 5, 6, 7, 8, 0,
 0, 1, 2, 3, 4, 5, 6, 7, 8, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

/* Valeur de base des pices du jeu d'checs : */
const int pieces_value[7] = { 0, 100, 300, 300, 500, 900, 0 };

/* Pour chaque case de l'chiquier, donne le numro de la range : */
const int rank[120] = {
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 1, 1, 1, 1, 1, 1, 1, 1, 0,
 0, 2, 2, 2, 2, 2, 2, 2, 2, 0,
 0, 3, 3, 3, 3, 3, 3, 3, 3, 0,
 0, 4, 4, 4, 4, 4, 4, 4, 4, 0,
 0, 5, 5, 5, 5, 5, 5, 5, 5, 0,
 0, 6, 6, 6, 6, 6, 6, 6, 6, 0,
 0, 7, 7, 7, 7, 7, 7, 7, 7, 0,
 0, 8, 8, 8, 8, 8, 8, 8, 8, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

/* Tableau qui donne pour chaque range de l'chiquier son rang par rapport
    la couleur : */
const int reverse_rank[2][9] = {
  { 0, 8, 7, 6, 5, 4, 3, 2, 1},
  { 0, 1, 2, 3, 4, 5, 6, 7, 8}
};

/* La couleur du camp qui a le trait :
   (Pour les valeurs possibles, cf "Noms des couleurs" dans data.h) : */
int active_color;

/* Temps de base (en ms) allou pour un coup, calcul au dbut de la partie : */
int base_time;

/* L'chiquier est reprsent par un tableau linaire de 120 cases :
   a1 -> 21, e1 -> 25, e4 -> 55, e8 -> 95, h8 -> 98.
  (voir la dfinition de la structure board_s)
{ 0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
  0, a1, b1, c1, d1, e1, f1, g1, h1,  0,
  0, a2, b2, c2, d2, e2, f2, g2, h2,  0,
  0, a3, b3, c3, d3, e3, f3, g3, h3,  0,
  0, a4, b4, c4, d4, e4, f4, g4, h4,  0,
  0, a5, b5, c5, d5, e5, f5, g5, h5,  0,
  0, a6, b6, c6, d6, e6, f6, g6, h6,  0,
  0, a7, b7, c7, d7, e7, f7, g7, h7,  0,
  0, a8, b8, c8, d8, e8, f8, g8, h8,  0,
  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 } */
int board[120];

/* Indique les possibilits de roque blanc et noir :
   - bit 1 petit roque blanc possible
   - bit 2 grand roque blanc possible
   - bit 3 petit roque noir possible
   - bit 4 grand roque noir possible
   Indique que le roi blanc a roqu :
   - bit  9 roque blanc effectu
   - bit 10 roque noir effectu

   Exemple : 0xF -> tous les roques sont possibles
             0x6 -> seuls le petit roque noir et le grand roque blanc sont
                    possibles
*/
int castle_status[MOVES_LIMIT * 2];

/* La couleur joue par le programme :
   (Pour les valeurs possibles, cf "Noms des couleurs" dans data.h) : */
int computer_color;

/* Profondeur de recherche atteinte (en demi-coups) : */
int depth;

/* Temps de rflexion coul en millimes de secondes. */
int elapsed_time_ms;

/* La case de prise en passant; elle est renseigne seulement si le coup
   prcdent correspond  l'avance d'un pion de deux cases : */
int ep_square[MOVES_LIMIT * 2];

/* This number is the fullmove number. It will have the value "1" for the first
   move of a game for both White and Black. It is incremented by one immediately
   after each move by Black : */
int fullmove;

/* This number is the count of halfmoves since the begin of the game. */
int game_ply;

/* This number is the count of halfmoves (or ply) since the last pawn advance
   or capturing move. It is used for the fifty move draw rule : */
int halfmove;
int halfmove_history[MOVES_LIMIT * 2];

/* Position des rois blancs et noirs : */
int king[2];

/* Profondeur de recherche maximum (permet principalement de donner une limite 
   la rflexion du programme dans) : */
int maximum_depth;

/* Profondeur de recherche maximum dans la recherche d'une situation calme : */
int maximum_quiet_depth;

/* The maximum amount of time (in ms) for the search : */
int maximum_search_time;

/* Profondeur de recherche minimum (permet d'assurer au programme un minimum de
   rflexion) : */
int minimum_depth;

/* Indique si une pice a t dplace : */
int moved[120];

/* Nombre de noeuds de l'arbre de recherche parcourus par le programme : */
int nodes;

/* Nombre de pices sur l'chiquier, blanches et noires, autres que pions et
   rois. Permet de connatre l'volution de ce nombre pendant la recherche : */
int piece_count;

/* Table des pices (16 pices au maximum pour chaque couleur) : */
int pieces[2][17];

/* Le nombre de pices de chaque couleur : */
int pieces_number[2];

/* Adresse de la fiche associe  la pice situe sur une case : */
int *pieces_ptr[120];

/* Longueur de la variante principale  chaque demi-coup de l'arbre de
   recherche : */
int pv_length[PV_LIMIT];

/* Nombre de noeuds parcourus lors de la phase de recherche des situations
   calmes pour valuer la position : */
int qnodes;

/* Mmorise le nombre de rptitions d'une position, depuis le dbut de la
   partie, avant son valuation pour viter des parties nulles lorsque le
   programme possde un net avantage. */
int repeat_counter;

/* Rsultat de la partie :
   (Pour les valeurs possibles, cf "Noms des rsultats" dans data.h) : */
int result;

/* This number is the count of halfmoves since the start of the search. */
int search_ply;

/* Nombre de pices sur l'chiquier (cf piece_count), au dbut de chaque
   nouvelle recherche du programme : */
int start_piece_count;

/* Temps allou pour la rflexion du programme (en millimes de seconde) : */
int time_for_move;

int time_in_console_mode;

/* Coup sans signification utilis au retour des fonctions lorsqu'aucun coup
   n'est  prendre en compte, ou pour initialiser des donnes : */
move_s null_move;

/* Liste des coups jous (en nombre de demi-coups) : */
move_s played_moves[MOVES_LIMIT * 2];

/* Liste des coups de la variante principale  chaque demi-coup de l'arbre de
   recherche : */
move_s pv[PV_LIMIT][PV_LIMIT];
