/*******************************************************************************
  File : data.h

  Description : Fichier en-tte pour les donnes globales.
*******************************************************************************/

#ifndef DATA_H
#define DATA_H

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des donnes gnrales du programme : */
#include "main.h"

/*******************************************************************************
  Macro-instructions :
*******************************************************************************/

#define CASTLE_STATUS castle_status[game_ply]
#define EP_SQUARE     ep_square[game_ply]
#define PIECE(x) \
  (x == BPAWN   || x == WPAWN   ? 'p' : \
   x == BBISHOP || x == WBISHOP ? 'b' : \
   x == BKNIGHT || x == WKNIGHT ? 'n' : \
   x == BROOK   || x == WROOK   ? 'r' : \
   x == BQUEEN  || x == WQUEEN  ? 'q' : \
   x == BKING   || x == WKING   ? 'k' : \
   ' ')
#define RR(c) (reverse_rank[color][c])

/*******************************************************************************
  Types :
*******************************************************************************/

/* Informations qui dcrivent un coup : */
typedef struct {

  /* Case de dpart de la pice : */
  short int from;

  /* Case d'arrive de la pice : */
  short int to;

  /* Identification du coup sur 24 bits :
     - bits  1   8 : case de dpart,
     - bits  9  16 : case d'arrive,
     - bits 17  24 : nom de la pice de promotion. */
  int moveid;

  /* Nature du coup jou : */
  short int status;

  /* Nom de la pice capture : */
  short int captured;

  /* Nom de la pice de promotion : */
  short int promoted;

  /* Note pour le tri des coups lors de la phase de recherche : */
  int note;

  /* Adresse de la fiche associe  la pice capture : */
  int *captured_ptr;

} move_s;

/*******************************************************************************
  Data :
 (Pour la description des donnes, voir le fichier data.c)
*******************************************************************************/

extern FILE *log_file;

extern bool book_use;
extern bool forced_mode;
extern bool log_use;
extern bool show_board;
extern bool search_on_depth_limit;
extern bool test_mode;
extern bool uci_mode;

extern char start_position[STRING_LENGTH];

extern clock_t start_thinking_time;

extern const int castle_by_color_mask[2];
extern const int castle_data_mask[7];
extern const int castle_done_mask[2];
extern const int castle_possible_mask[2];
extern const int file[120];
extern const int pieces_value[7];
extern const int rank[120];
extern const int reverse_rank[2][9];

extern int active_color;
extern int base_time;
extern int board[120];
extern int castle_status[MOVES_LIMIT * 2];
extern int computer_color;
extern int depth;
extern int elapsed_time_ms;
extern int ep_square[MOVES_LIMIT * 2];
extern int fullmove;
extern int game_ply;
extern int halfmove;
extern int halfmove_history[MOVES_LIMIT * 2];
extern int king[2];
extern int maximum_depth;
extern int maximum_quiet_depth;
extern int maximum_search_time;
extern int minimum_depth;
extern int moved[120];
extern int nodes;
extern int piece_count;
extern int pieces[2][17];
extern int pieces_number[2];
extern int *pieces_ptr[120];
extern int pv_length[PV_LIMIT];
extern int qnodes;
extern int repeat_counter;
extern int result;
extern int search_ply;
extern int start_piece_count;
extern int time_for_move;
extern int time_in_console_mode;

extern move_s null_move;
extern move_s played_moves[MOVES_LIMIT * 2];
extern move_s pv[PV_LIMIT][PV_LIMIT];

#endif /* DATA_H */
