/*******************************************************************************
  File : debug.c

  Description : Diverses fonctions utilises pour dboguer ou tester le
                programme.
*******************************************************************************/

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des diffrents modules du programme : */
#include "debug.h"
#include "hash.h"
#include "moves.h"
#include "tools.h"
#include "uci.h"

/*******************************************************************************
  Data :
*******************************************************************************/

/* Fichier pour enregistrer les informations du programme : */
FILE *dump_file;

#ifdef DEBUG
/* Fichier pour enregistrer les informations de dbogage du module
   d'valuation : */
FILE *debug_eval_file;
#endif /* DEBUG */

#ifdef DEBUG
/* Fichier pour enregistrer les informations de dbogage : */
FILE *debug_file;
#endif /* DEBUG */

#ifdef DEBUG
/* Fichier pour enregistrer les informations de dbogage du module
   de recherche : */
FILE *debug_tree_file;
#endif /* DEBUG */

#ifdef DEBUG
/* Variables de comptage des appels aux diffrentes fonctions du programme :
   ------------------------------------------------------------------------- */

/* Du module : book */
int select_move_calls = 0;
int make_book_calls = 0;

/* Du module : evaluate */
int check_game_phase_calls = 0;
int evaluate_position_calls = 0;

/* Du module : hash */
int compute_h_position_calls = 0;
int init_h_values_calls = 0;
int update_h_position_calls = 0;

/* Du module : main */
int call_uci_function_calls = 0;
int exec_cmd_calls = 0;
int exec_dbg_cmd_calls = 0;
int init_game_calls = 0;
int init_program_calls = 0;
int main_calls = 0;
int start_thinking_calls = 0;

/* Du module : moves */
int generate_calls = 0;
int is_attacked_calls = 0;
int is_legal_calls = 0;
int make_calls = 0;
int unmake_calls = 0;

/* Du module : search */
int apply_heuristic_calls = 0;
int pick_move_calls = 0;
int quiescence_search_calls = 0;
int search_calls = 0;
int sort_moves_calls = 0;
int think_calls = 0;
int update_killers_calls = 0;

/* Du module : tools */
int compute_time_calls = 0;
int convert_lan_move_calls = 0;
int convert_pgm_move_calls = 0;
int generate_random_calls = 0;
int is_repeated_calls = 0;
int print_board_calls = 0;
int read_input_calls = 0;
int send_output_calls = 0;
int update_pieces_calls = 0;
int verify_move_calls = 0;

/* Du module : uci */
int go_calls = 0;
int info_calls = 0;
int isready_calls = 0;
int position_calls = 0;
int quit_calls = 0;
int uci_cmd_calls = 0;
int ucinewgame_calls = 0;

/* Autres variables de dbogage. */
bool debug_evaluation;
bool debug_tree;

char debug_file_name[STRING_LENGTH];

int debug_file_indice = 1;
int debug_node;
int total_calls = 0;
#endif /* DEBUG */

/*******************************************************************************
  External functions :
*******************************************************************************/

/*******************************************************************************
  Function     : close_debug_file
  Description  : Fermeture du fichier de debogage.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void close_debug_file (void) {

#ifdef DEBUG
  fclose (debug_file);
#endif /* DEBUG */
}
/* End of function : close_debug_file */

/*******************************************************************************
  Function     : dump
  Description  : Enregistre dans un fichier les donnes du programme.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void dump (void) {

/* ------------------------------- Local data ------------------------------- */

int color;    /* Couleur du camp. */
int i;        /* Indice de boucle. */
int x, y;     /* Colonne et range de l'chiquier. */
int square;   /* Case de l'chiquier. */

/* ------------------------------ Function code ----------------------------- */

  /* Ouverture du fichier. */
  if ((dump_file = fopen (DUMP_FILE, "w")) == NULL) {

    tools.send_output (true, "\n   ERROR : %s can't open the debug file %s\n",
      PRODUCT_NAME, DUMP_FILE);
  }
  else {

    /* Impression de la position courante. */
    fprintf (dump_file, "Position courante :\n");
    tools.print_board (dump_file);

    /* Listage des coups de la partie. */
    fprintf (dump_file, "\nListe des coups de la partie :\n");
    for (i = 0; i < game_ply; i++) {
      fprintf (dump_file, " %c%d%c%d%c",
        file[played_moves[i].from] + (int) 'a' - 1, rank[played_moves[i].from],
        file[played_moves[i].to] + (int) 'a' - 1, rank[played_moves[i].to],
        played_moves[i].status == PROMOTION ?
          PIECE(played_moves[i].promoted) : ' ');
    }
    fprintf (dump_file, "\n");

    /* Enregistrement des donnes globales : */
    fprintf (dump_file, "\nValeurs des donnes globales :\n");
    fprintf (dump_file, "active_color       : %d\n", active_color);
    fprintf (dump_file, "base_time          : %d\n", base_time);
    fprintf (dump_file, "castle_status[%2d]  : %x\n",
      game_ply, castle_status[game_ply]);
    fprintf (dump_file, "computer_color     : %d\n", computer_color);
    fprintf (dump_file, "depth              : %d\n", depth);
    if (ep_square[game_ply] != 0) {
      fprintf (dump_file, "ep_square[%2d]      : %c%d\n", game_ply,
        file[ep_square[game_ply]] + (int) 'a' - 1, rank[ep_square[game_ply]]);
    }
    else {
      fprintf (dump_file, "ep_square[%2d]      : none\n", game_ply);
    }
    fprintf (dump_file, "forced_mode        : %d\n", forced_mode);
    fprintf (dump_file, "fullmove           : %d\n", fullmove);
    fprintf (dump_file, "game_ply           : %d\n", game_ply);
    fprintf (dump_file, "halfmove           : %d\n", halfmove);
    fprintf (dump_file, "h_position         : %8x %8x\n",
      h_position.h1, h_position.h2);
    fprintf (dump_file, "king position      : black %c%d - white %c%d\n",
      file[king[BLACK]] + (int) 'a' - 1, rank[king[BLACK]],
      file[king[WHITE]] + (int) 'a' - 1, rank[king[WHITE]]);
    fprintf (dump_file, "nodes              : %d\n", nodes);
    fprintf (dump_file, "piece_count        : %d\n", piece_count);
    fprintf (dump_file, "pieces_number      : black %2d / white %2d\n",
      pieces_number[BLACK], pieces_number[WHITE]);
    fprintf (dump_file, "qnodes             : %d\n", qnodes);
    fprintf (dump_file, "result             : %d\n", result);
    fprintf (dump_file, "search_ply         : %d\n", search_ply);
    fprintf (dump_file, "show_board         : %d\n", show_board);
    fprintf (dump_file, "start_piece_count  : %d\n", start_piece_count);
    fprintf (dump_file, "uci_mode           : %d\n", uci_mode);

    /* Enregistrement de la table des pices : */
    fprintf (dump_file, "\nContenu de la table des pices :\n");
    for (color = BLACK; color <= WHITE; color++) {

      for (i = 1; i <= pieces_number[color]; i++) {

        fprintf (dump_file, "pieces[%d][%2d] %c%d - adress %8x\n",
          color, i,
          file[pieces[color][i]] + (int) 'a' - 1,
          rank[pieces[color][i]], (unsigned int) &pieces[color][i]);
      }
    }

    /* Enregistrement de l'chiquier : */
    fprintf (dump_file, "\nContenu de la table de l'chiquier :\n");
    for (y = 1; y <= 8; y++) {

      for (x = A; x <= H; x++) {

        square = (y + 1) * 10 + x;
        fprintf (dump_file, "board[%c%d] %2d pieces_ptr %8x moved %2d\n",
          x + (int) 'a' - 1, y, board[square],
          (unsigned int) pieces_ptr[square], moved[square]);
      }
    }

    tools.send_output (true, "\n   Data are written in %s.\n", DUMP_FILE);

    /* Enregistrement des valeurs de hachage : */
    fprintf (dump_file,
      "\nHistorique des valeurs de hachage des positions :\n");
    for (i = 0; i <= game_ply; i++) {

      fprintf (dump_file, "h_position[%2d]     : %8x %8x\n",
        i, h_position_history[i].h1, h_position_history[i].h2);
    }

    /* Fermeture du fichier. */
    fclose (dump_file);
  }
}
/* End of function : dump */

/*******************************************************************************
  Function     : open_debug_file
  Description  : Ouverture des fichiers utiliss pour le dbogage du programme.
                 Cela concerne le dbogage gnral avec l'enchanement des
                 fonctions.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void open_debug_file (void) {

/* ------------------------------- Local data ------------------------------- */

#ifdef DEBUG
char string_indice[4];       /* Pour numroter les fichiers de dbogage. */
#endif /* DEBUG */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
  sprintf (debug_file_name, DEBUG_FILE);
  sprintf (string_indice, "%d", debug_file_indice);
  strcat (debug_file_name, string_indice);
  strcat (debug_file_name, ".txt");

/*
  if (debug_file_indice > 1) {

    printf ("                %8d : %s\n", nodes, debug_file_name);
  }
*/

  if ((debug_file = fopen (debug_file_name, "w")) == NULL) {

    printf ("   ERROR : %s can't open the debug file %s\n",
      PRODUCT_NAME, debug_file_name);

    uci.quit (EXIT_FAILURE);
  }
#endif /* DEBUG */
}
/* End of function : open_debug_file */

/*******************************************************************************
  Function     : print
  Description  : Pour enregistrer des informations dans le fichier de dbogage.
                 Afin de limiter le volume d'informations, les critures seront
                 bornes entre deux valeurs de noeuds parcourus dans l'arbre de
                 recherche.
  Parameters   : in  - format d'impression
                 in  - liste des donnes  imprimer
  Return value : nombre de caractres crits
  Validation   : 1
*******************************************************************************/
int print (const char *format, ...) {

/* ------------------------------- Local data ------------------------------- */

int size = 0;                /* Nombre de caractres crits. */

#ifdef DEBUG
va_list list;                /* Liste variable des arguments. */
#endif /* DEBUG */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
  if (nodes >= DEBUG_NODES_MIN && nodes < DEBUG_NODES_MAX) {

    /* Initialisation de la liste des paramtres : */
    va_start (list, format);

    /* Ecriture des donnes dans le fichier de dbogage : */
    size = vfprintf (debug_file, format, list);
    fflush (debug_file);

    /* Fin de lecture de la liste des paramtres : */
    va_end (list);
  }
#endif /* DEBUG */

  return size;
}
/* End of function : print */

/*******************************************************************************
  Function     : print_informations
  Description  : Donne des informations sur l'environnement d'excution du
                 programme.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void print_informations (void) {

  if (log_use) {

    fprintf (log_file, "\nInformations about ANSI C standard types :\n");
    fprintf (log_file, "--------------------------------------------\n");
    fprintf (log_file, "char                   : %d (%d,%d)\n",
             sizeof (char), CHAR_MIN, CHAR_MAX);
    fprintf (log_file, "short                  : %d (%d,%d)\n",
             sizeof (short), SHRT_MIN, SHRT_MAX);
    fprintf (log_file, "int                    : %d (%d,%d)\n",
             sizeof (int), INT_MIN, INT_MAX);
    fprintf (log_file, "long int               : %d (%ld,%ld)\n",
             sizeof (long int), LONG_MIN, LONG_MAX);
    fprintf (log_file, "unsigned char          : %d (%d,%d)\n",
             sizeof (unsigned char), 0, UCHAR_MAX);
    fprintf (log_file, "unsigned short         : %d (%d,%d)\n",
             sizeof (unsigned short), 0, USHRT_MAX);
    fprintf (log_file, "unsigned int           : %d (%d,%u)\n",
             sizeof (unsigned int), 0, UINT_MAX);
    fprintf (log_file, "unsigned long int      : %d (%d,%lu)\n",
             sizeof (unsigned long int), 0, ULONG_MAX);
    fprintf (log_file, "float                  : %d\n", sizeof (float));
    fprintf (log_file, "double                 : %d\n", sizeof (double));
    fprintf (log_file, "\n");
  }
}
/* End of function : print_informations */

/*******************************************************************************
  Function     : print_statistics
  Description  : Donne des informations statistiques sur les appels de fonction.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void print_statistics (void) {

#ifdef DEBUG
  if (log_use) {

    fprintf (log_file, "Statistics about function calls :");
    fprintf (log_file, "\n---------------------------------\n");
    fprintf (log_file, "\n*******************************\n");
    fprintf (log_file, "* book module                 *\n");
    fprintf (log_file, "*******************************\n");
    fprintf (log_file, "select_move         : %9d\n", select_move_calls);
    fprintf (log_file, "make_book           : %9d\n", make_book_calls);
    fprintf (log_file, "\n*******************************\n");
    fprintf (log_file, "* evaluate module             *\n");
    fprintf (log_file, "*******************************\n");
    fprintf (log_file, "check_game_phase    : %9d\n", check_game_phase_calls);
    fprintf (log_file, "evaluate_position   : %9d\n", evaluate_position_calls);
    fprintf (log_file, "\n*******************************\n");
    fprintf (log_file, "* hash module                 *\n");
    fprintf (log_file, "*******************************\n");
    fprintf (log_file, "compute_h_position  : %9d\n", compute_h_position_calls);
    fprintf (log_file, "initialize_h_values : %9d\n", init_h_values_calls);
    fprintf (log_file, "update_h_position   : %9d\n", update_h_position_calls);
    fprintf (log_file, "\n*******************************\n");
    fprintf (log_file, "* main module                 *\n");
    fprintf (log_file, "*******************************\n");
    fprintf (log_file, "call_uci_function   : %9d\n", call_uci_function_calls);
    fprintf (log_file, "execute_command     : %9d\n", exec_cmd_calls);
    fprintf (log_file, "execute_debug_com.  : %9d\n", exec_dbg_cmd_calls);
    fprintf (log_file, "initialize_game     : %9d\n", init_game_calls);
    fprintf (log_file, "initialize_program  : %9d\n", init_program_calls);
    fprintf (log_file, "main                : %9d\n", main_calls);
    fprintf (log_file, "start_thinking      : %9d\n", start_thinking_calls);
    fprintf (log_file, "\n*******************************\n");
    fprintf (log_file, "* moves module                *\n");
    fprintf (log_file, "*******************************\n");
    fprintf (log_file, "generate            : %9d\n", generate_calls);
    fprintf (log_file, "is_attacked         : %9d\n", is_attacked_calls);
    fprintf (log_file, "is_legal            : %9d\n", is_legal_calls);
    fprintf (log_file, "make                : %9d\n", make_calls);
    fprintf (log_file, "unmake              : %9d\n", unmake_calls);
    fprintf (log_file, "\n*******************************\n");
    fprintf (log_file, "* search module               *\n");
    fprintf (log_file, "*******************************\n");
    fprintf (log_file, "apply_heuristic     : %9d\n", apply_heuristic_calls);
    fprintf (log_file, "pick_move           : %9d\n", pick_move_calls);
    fprintf (log_file, "quiescence_search   : %9d\n", quiescence_search_calls);
    fprintf (log_file, "search              : %9d\n", search_calls);
    fprintf (log_file, "sort_moves          : %9d\n", sort_moves_calls);
    fprintf (log_file, "think               : %9d\n", think_calls);
    fprintf (log_file, "update_killers      : %9d\n", update_killers_calls);
    fprintf (log_file, "\n*******************************\n");
    fprintf (log_file, "* tools module                *\n");
    fprintf (log_file, "*******************************\n");
    fprintf (log_file, "compute_time        : %9d\n", compute_time_calls);
    fprintf (log_file, "convert_lan_move    : %9d\n", convert_lan_move_calls);
    fprintf (log_file, "convert_pgm_move    : %9d\n", convert_pgm_move_calls);
    fprintf (log_file, "generate_random     : %9d\n", generate_random_calls);
    fprintf (log_file, "is_repeated         : %9d\n", is_repeated_calls);
    fprintf (log_file, "print_board         : %9d\n", print_board_calls);
    fprintf (log_file, "read_input          : %9d\n", read_input_calls);
    fprintf (log_file, "send_output         : %9d\n", send_output_calls);
    fprintf (log_file, "update_pieces       : %9d\n", update_pieces_calls);
    fprintf (log_file, "verify_move         : %9d\n", verify_move_calls);
    fprintf (log_file, "\n*******************************\n");
    fprintf (log_file, "* uci module                  *\n");
    fprintf (log_file, "*******************************\n");
    fprintf (log_file, "go                  : %9d\n", go_calls);
    fprintf (log_file, "info                : %9d\n", info_calls);
    fprintf (log_file, "isready             : %9d\n", isready_calls);
    fprintf (log_file, "position            : %9d\n", position_calls);
    fprintf (log_file, "quit                : %9d\n", quit_calls);
    fprintf (log_file, "uci_command         : %9d\n", uci_cmd_calls);
    fprintf (log_file, "ucinewgame          : %9d\n", ucinewgame_calls);
    fprintf (log_file, "\n*******************************\n");
    fprintf (log_file, "Total calls         : %9d\n", total_calls);
  }
#endif /* DEBUG */
}
/* End of function : print_statistics */

/*******************************************************************************
  Function     : split_debug_file
  Description  : En raison du volume important des donnes, le fichier de
                 dbogage est rgulirement dcoup en fonction du nombre
                 de noeuds parcourus lors de la recherche.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void split_debug_file (void) {

#ifdef DEBUG
  if (nodes > DEBUG_NODES_MIN && nodes <= DEBUG_NODES_MAX) {

    /* Pour chaque noeud parcouru, si la limite pour un fichier de dbogage est
       atteinte, */
    if (nodes % DEBUG_NODES_LIMIT == 0) {

      /* alors on repart pour un nouveau cycle avec un nouveau fichier de
         dbogage. */
      debug_file_indice++;

      debug.close_debug_file ();
      debug.open_debug_file ();
    }
  }
#endif /* DEBUG */
}
/* End of function : split_debug_file */

/*******************************************************************************
  Function     : test_hash
  Description  : Permet de tester les collisions sur les valeurs de hachage.
                 L'ide est de vrifier que dans une partie, la valeur de
                 hachage d'une position est identique uniquement pour des
                 positions identiques.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void test_hash (void) {

/* ------------------------------- Local data ------------------------------- */

FILE *games_file;            /* Fichier contenant les parties  tester. */
FILE *hash_test_file;        /* Fichier de rsultat du test. */

bool legal_move;             /* Indique la lgalit d'un coup. */
bool white_move;             /* Indique un coup blanc en cours de traitement. */
bool end_of_game;            /* Indique la fin d'une partie. */

char game[STRING_LENGTH];    /* Partie lue dans le fichier "games_files". */
char lan_move[6];            /* Coup aprs conversion en notation algbrique. */
char input_lan_move[6];      /* Coup lu dans le fichier des parties. */

int count_games = 0;         /* Nombre de parties traites. */
int i, j, k;                 /* Indices de boucle. */
int collisions = 0;          /* Nombre de collisions des valeurs de hachage. */
int moves_number;            /* Nombre de coups jouables. */
int position_id[MOVES_LIMIT * 2][11];  /* Ensemble de valeurs numriques pour
                                          identifier une position. */
int x, y;                    /* Colonne et range de l'chiquier. */

move_s moves_list[MOVES_LIMIT];   /* Pour ranger les coups d'une position. */

/* ------------------------------ Function code ----------------------------- */

  if (   (games_file = fopen (GAMES_FILE, "r")) == NULL
      || (hash_test_file = fopen (HASH_TEST_FILE, "w")) == NULL) {

    tools.send_output (true,
      "\n   ERROR : %s can't open the games and hash test file\n"
      "           (%s et/ou %s).\n\n", PRODUCT_NAME, GAMES_FILE,
      HASH_TEST_FILE);
  }
  else {

    printf ("\n   Each point equal 100 games, a line 5000 games.\n   ");
    while (!feof (games_file)) {

      count_games++;

      if ((count_games % 100) == 0) {
        printf (".");
        if ((count_games % 5000) == 0) {
          printf ("\n   ");
        }
      }

      /* Les 4 premires lignes correspondent  l'en-tte d'une partie et ne
         sont pas conserves : */
      for (i = 0; i < 5; i++) {

        fgets (game, STRING_LENGTH, games_file);
      }

      i = 0;
      white_move = true;
      end_of_game = false;
      while (!end_of_game) {

        if (white_move) {
          /* Saut du numro du coup : */
          while ((game[i++]) != (int) '.');
        }
        else {
          /* Saute au coup des noirs : */
          i++;
        }

        input_lan_move[4] = '\0';
        input_lan_move[5] = '\0';
        /* Lecture du coup : */
        j = 0;
        while (game[i] != (int) ' ' && i < strlen (game)) {
          if (   (game[i] >= '0' && game[i] <= '8')
              || (game[i] >= 'a' && game[i] <= 'r')) {
            input_lan_move[j++] = game[i];
          }
          if (j > 3) {
            switch (game[i]) {
              case 'D' : case 'Q' : input_lan_move[j] = 'q'; break;
              case 'T' : case 'R' : input_lan_move[j] = 'r'; break;
              case 'F' : case 'B' : input_lan_move[j] = 'b'; break;
              case 'C' : case 'N' : input_lan_move[j] = 'n'; break;
            }
          }
          i++;
        }

        if (i < strlen (game)) {

          /* Cas particulier des roques : */
          if (!strncmp (input_lan_move, "000", 3)) {
            if (white_move) {
              strcpy (input_lan_move, "e1c1");
            }
            else {
              strcpy (input_lan_move, "e8c8");
            }
          }
          else if (!strncmp (input_lan_move, "00", 2)) {
            if (white_move) {
              strcpy (input_lan_move, "e1g1");
            }
            else {
              strcpy (input_lan_move, "e8g8");
            }
          }

          /* Recherche la liste des coups jouables dans la position courante. */
          moves_number = moves.generate (moves_list, false);

          legal_move = false;

          /* Parcourt la liste obtenue. */
          do {

            /* Conversion du coup pour permettre la comparaison. */
            tools.convert_pgm_move (&moves_list[moves_number], lan_move);

            /* Si le coup entr correspond  un coup jouable, */
            if (!strcmp (input_lan_move, lan_move)) {

              /* alors il est jou sur l'chiquier, */
              moves.make (&moves_list[moves_number]);

              /* pour vrifier sa lgalit. */
              if (moves.is_legal (&moves_list[moves_number])) {

                legal_move = true;

                /* Mmorise les caractristiques de la position courante sans
                   risque de collision ou perte d'information. Le tableau
                   contiendra toutes les caractristiques d'une position sur 9
                   entiers de 4 octets. */
                memset (position_id [game_ply], 0,
                  sizeof (position_id [game_ply]));

                /* Mmorise la couleur au trait, l'tat des roques et une
                   ventuelle prise en passant sur un entier de 4 octets. */
                position_id [game_ply][0] = active_color;
                position_id [game_ply][0] = (position_id [game_ply][0] << 4)
                  + (castle_status [game_ply] & 0xf);
                position_id [game_ply][0] = (position_id [game_ply][0] << 4)
                  + ep_square [game_ply];

                /* Mmorise pour chaque range, sur un entier sur 4 octets, les
                   pices prsentes sur chaque case. */
                for (x = A; x <= H; x++) {

                  for (y = 1; y <= 8; y++) {

                    position_id[game_ply][y] = (position_id[game_ply][y] << 4)
                      + board [(y + 1) * 10 + x];
                  }
                }

                /* Pour tous les coups de la partie, */
                for (j = 0; j < game_ply; j++) {

                  /* si la valeur de hachage pour une position se retrouve dans
                     les valeurs mmorises pour les positions prcdentes, */
                  if (   h_position_history[j].h1 == h_position.h1
                      && h_position_history[j].h2 == h_position.h2) {

                    /* le programmme va vrifier que les toutes les informations
                       de la position mmorises dans le tableau position_id
                       sont identiques. */
                    for (k = 0; k < 9; k++) {

                      /* Si une information est diffrente entre les deux
                         positions, */
                      if (position_id [game_ply][k] != position_id [j][k]) {

                        /* alors cela signifie que deux valeurs de hachage
                           identiques peuvent apparatre pour deux positions
                           diffrentes. Il y a alors une collision. */
                        collisions++;

                        fprintf (hash_test_file, "game %4d : %s",
                          count_games, game);

                        fprintf (hash_test_file,
                          "game_ply %3d - hposition %8x %8x :", j,
                          h_position_history[j].h1, h_position_history[j].h2);

                        for (k = 0; k < 9; k++) {

                          fprintf (hash_test_file, "%8x,", position_id[j][k]);
                        }
                        fprintf (hash_test_file, "\n");

                        fprintf (hash_test_file,
                          "game_ply %3d - hposition %8x %8x :",
                          game_ply, h_position.h1, h_position.h2);

                        for (k = 0; k < 9; k++) {

                          fprintf (hash_test_file, "%8x,",
                            position_id[game_ply][k]);
                        }
                        fprintf (hash_test_file, "\n\n");
                      }
                    }
                  }
                }
              }
            }
          } while (--moves_number && !legal_move);

          if (!legal_move) {

            /* Le coup en cours de traitement ne semble pas correct. */
            fprintf (hash_test_file, "\n   ERROR : Invalid move. "
              "(Game %d, %d, %s)\n", count_games, game_ply + 1, input_lan_move);
            fprintf (hash_test_file, "%s\n",game);

            /* Le programme passe  la partie suivante. */
            break;
          }

          /* Aprs la lecture du coup des blancs, passage au coup des noirs et
             vice versa. */
          white_move = (white_move ? false : true);
        }
        else {

           /* La fin de la partie est atteinte. */
           end_of_game = true;
        }
      }

      /* Rinitialisation des informations pour passage  la partie suivante. */
      uci.ucinewgame ();
    }
    printf ("\n");

    /* Fermeture des fichiers utiliss. */
    fclose (games_file);
    fclose (hash_test_file);

    /* Affichage du rsultat du test. */
    tools.send_output (true, "\n   %s has detected %d collision(s) for %d"
      " games.\n   The result is written in the hash test file %s.\n",
      PRODUCT_NAME, collisions, count_games, HASH_TEST_FILE);
  }
}
/* End of function : test_hash */

/*******************************************************************************
  Function     : trace
  Description  : Pour tracer l'appel  chaque fonction du programme. Tous les
                 appels sont comptabiliss, mais pour limiter le volume
                 d'informations  analyser, seul l'appel  quelques fonctions
                 sera indiqu dans le fichier de dbogage.
  Parameters   : in     - nom de la fonction
                 in     - indique si l'appel doit tre crit dans le fichier de
                          dbogage
                 in/out - nombre d'appels  la fonction
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void trace (char function[], bool is_printed, int *number_of_calls) {

#ifdef DEBUG
  total_calls++;

  if (is_printed) {

/*
    debug.print ("%9d (%9d) - %20s", total_calls, nodes, function);
*/
    debug.print ("(%9d) - %20s", nodes, function);
  }

  (*number_of_calls)++;
#endif /* DEBUG */
}
/* End of function : trace */

/*******************************************************************************
  Object declaration :
*******************************************************************************/

debug_module_s debug = {

  close_debug_file,
  dump,
  open_debug_file,
  print,
  print_informations,
  print_statistics,
  split_debug_file,
  test_hash,
  trace
};
