/*******************************************************************************
  File : debug.h

  Description : Fichier en-tte pour le module "debug".
*******************************************************************************/

#ifndef DEBUG_H
#define DEBUG_H

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des informations gnrales du programme : */
#include "main.h"

/* Fichier en-tte des donnes globales : */
#include "data.h"

/*******************************************************************************
  Constants :
*******************************************************************************/

/* Nom des fichiers utiliss pour le dbogage du programme : */
#define DEBUG_EVAL_FILE      "log/debug_eval.txt"
#define DEBUG_FILE           "log/debug"
#define DEBUG_TREE_FILE      "log/debug_tree.txt"
#define DUMP_FILE            "log/dump_file.txt"
#define HASH_TEST_FILE       "log/hash_test.txt"

/* Valeurs limites pour quelques constantes gnrales : */
#define DEBUG_NODES_LIMIT     10000
#define DEBUG_NODES_MAX       50000
#define DEBUG_NODES_MIN           0
#define DEBUG_TREE_LIMIT    2000000

/*******************************************************************************
  Macros :
*******************************************************************************/

/* Du module : evaluate */

#ifdef DEBUG
#define DB_EVAL(p1, p2) \
  if (debug_evaluation) \
    fprintf (debug_eval_file, "%c %-6s %c%d, score %5d, %-s\n", \
      color ? 'w' : 'b', p1 , x + (int) 'a' - 1, y, score, p2);
#endif /* DEBUG */

#ifdef DEBUG
#define DB_EVAL_1 \
  if (debug_evaluation) { \
    for (color = BLACK; color <= WHITE; color++) { \
      for (j = 1; j <= 8; j++) { \
        fprintf (debug_eval_file, "pawns[%d][%d] %d - pawn_rank[%d][%d] %d\n", \
          color, j, pawns[color][j], color, j, pawn_rank[color][j]); \
      } \
    } \
  }
#endif /* DEBUG */

/* Du module generate */

#ifdef DEBUG
#define DB_GENERATE_1 \
  debug.print (" : type of search %d\n", captures_only); \
  if (nodes >= DEBUG_NODES_MIN && nodes < DEBUG_NODES_MAX) { \
    tools.print_board (debug_file); \
  }
#endif /* DEBUG */

#ifdef DEBUG
#define DB_GENERATE_2 \
  debug.print ("moves (%d) :", moves_number); \
  for (i = 1; i <= moves_number; i++) { \
    debug.print (" %c%d%c%d%c", \
      file[moves[i].from] + (int) 'a' -1, rank[moves[i].from], \
      file[moves[i].to] + (int) 'a' -1, rank[moves[i].to], \
      moves[i].status == PROMOTION ? PIECE(moves[i].promoted) : ' '); \
  } \
  debug.print ("\n");
#endif /* DEBUG */

/* Du module moves */

#ifdef DEBUG
#define DB_MAKE_1 \
  debug.print (" : %d) %s%c%d%c%d%c\n", \
    game_ply, active_color ? "" : " ..... ", \
    file[move->from] + (int) 'a' - 1, rank[move->from], \
    file[move->to] + (int) 'a' - 1, rank[move->to], \
    move->status == PROMOTION ? PIECE(move->promoted) : ' '); \
  debug.print (" make history :"); \
  for (i = 0; i < game_ply; i++) { \
    debug.print (" %c%d%c%d%c", \
      file[played_moves[i].from] + (int) 'a' - 1, rank[played_moves[i].from], \
      file[played_moves[i].to] + (int) 'a' - 1, rank[played_moves[i].to], \
      played_moves[i].status == PROMOTION ? \
        PIECE(played_moves[i].promoted) : ' '); \
  }
#endif /* DEBUG */

#ifdef DEBUG
#define DB_MAKE_2 \
{ \
h_value_s tmp_h_position; \
 \
  tmp_h_position = hash.compute_h_position (); \
  debug.print (" - new h_position %8x %8x (from hash module %8x %8x)\n", \
    h_position.h1, h_position.h2, tmp_h_position.h1, tmp_h_position.h2); \
}
#endif /* DEBUG */

#ifdef DEBUG
#define DB_UNMAKE \
  debug.print (" : %d) %s%c%d%c%d%c\n", \
    game_ply, active_color ? " ..... " : "", \
    file[move->from] + (int) 'a' - 1, rank[move->from], \
    file[move->to] + (int) 'a' - 1, rank[move->to], \
    move->status == PROMOTION ? PIECE(move->promoted) : ' ');
#endif /* DEBUG */

/* Du module : search */

#ifdef DEBUG
#define DB_HISTORY_HEURISTIC(p1) \
  debug.print ("%s : history_heuristic[%c%d][%c%d] %d (+ %d)\n", p1, \
    file[moves_list[i].from] + (int) 'a' - 1, rank[moves_list[i].from], \
    file[moves_list[i].to] + (int) 'a' - 1, rank[moves_list[i].to], \
    history_heuristic[moves_list[i].from][moves_list[i].to], \
    live_depth);
#endif /* DEBUG */

#ifdef DEBUG
#define DB_KILLER_MOVES \
  debug.print ("search_ply %d - killer1 : score1 %d move1 %c%d%c%d%c", \
    search_ply, killers[search_ply].score1, \
    file[killers[search_ply].moveid1 & 0x7f] + (int) 'a' - 1, \
    rank[killers[search_ply].moveid1 & 0x7f], \
    file[killers[search_ply].moveid1 & 0x7f00] + (int) 'a' - 1, \
    rank[killers[search_ply].moveid1 & 0x7f00], \
    (killers[search_ply].moveid1 & 0x7f0000) ? \
      PIECE((killers[search_ply].moveid1 & 0x7f0000)) : ' '); \
  debug.print (" - score2 %d move2 %c%d%c%d%c", \
    killers[search_ply].score2, \
    file[killers[search_ply].moveid2 & 0x7f] + (int) 'a' - 1, \
    rank[killers[search_ply].moveid2 & 0x7f], \
    file[killers[search_ply].moveid2 & 0x7f00] + (int) 'a' - 1, \
    rank[killers[search_ply].moveid2 & 0x7f00], \
    (killers[search_ply].moveid2 & 0x7f0000) ? \
      PIECE((killers[search_ply].moveid2 & 0x7f0000)) : ' '); \
  debug.print (" - move3 %c%d%c%d%c\n", \
    file[killers[search_ply].moveid3 & 0x7f] + (int) 'a' - 1, \
    rank[killers[search_ply].moveid3 & 0x7f], \
    file[killers[search_ply].moveid3 & 0x7f00] + (int) 'a' - 1, \
    rank[killers[search_ply].moveid3 & 0x7f00], \
    (killers[search_ply].moveid3 & 0x7f0000) ? \
      PIECE((killers[search_ply].moveid3 & 0x7f0000)) : ' ');
#endif /* DEBUG */

#ifdef DEBUG
#define DB_SORTED_MOVES \
  debug.print (" : "); \
  for (i = 1; i <= moves_number; i++) { \
    debug.print ("%c%d%c%d%c %d - ", \
      file[moves_list[i].from] + (int) 'a' -1, rank[moves_list[i].from], \
      file[moves_list[i].to] + (int) 'a' -1, rank[moves_list[i].to], \
      moves_list[i].status == PROMOTION ? \
      PIECE(moves_list[i].promoted) : ' ', \
      moves_list[i].note); \
  } \
  debug.print ("\n");
#endif /* DEBUG */

#ifdef DEBUG
#define DB_TREE_1 \
  if (debug_tree) { \
 \
  int i, node; \
 \
    for (node = 1; node < nodes && node < DEBUG_TREE_LIMIT; node++) { \
      fprintf (debug_tree_file, "%6d %c%d ", \
        node, tree_nodes[node].type, tree_nodes[node].depth); \
      for (i = 0; i < tree_nodes[node].depth; i++) { \
        fprintf (debug_tree_file, "   "); \
      } \
      fprintf (debug_tree_file, "%2d/%2d - %s = %4d (alpha %8d beta %8d)\n", \
        tree_nodes[node].i, tree_nodes[node].moves_number, \
        tree_nodes[node].lan_move, tree_nodes[node].score, \
        tree_nodes[node].alpha, tree_nodes[node].beta); \
    } \
    fprintf (debug_tree_file, "\nTotal analyzed nodes of tree : %d\n", nodes); \
  }
#endif /* DEBUG */

#ifdef DEBUG
#define DB_TREE_2(p1,p2) \
  if (debug_tree) { \
    if (nodes < DEBUG_TREE_LIMIT) { \
      tools.convert_pgm_move (&moves_list[i], tree_nodes[nodes].lan_move); \
 \
      tree_nodes[nodes].type         = p1; \
      tree_nodes[nodes].depth        = p2; \
      tree_nodes[nodes].i            = i; \
      tree_nodes[nodes].moves_number = moves_number; \
      tree_nodes[nodes].alpha        = alpha; \
      tree_nodes[nodes].beta         = beta; \
 \
      debug_node = nodes; \
    } \
  }
#endif /* DEBUG */

#ifdef DEBUG
#define DB_TREE_3 \
  if (debug_tree && nodes < DEBUG_TREE_LIMIT) { \
    tree_nodes[debug_node].score = score; \
  }
#endif /* DEBUG */

/*******************************************************************************
  Types :
*******************************************************************************/

#ifdef DEBUG
/* Informations ncessaires au dbogage de l'arbre de recherche. */
struct {
  char type;
  int depth;
  int i;
  int moves_number;
  char lan_move[6];
  int alpha;
  int beta;
  int score;
} tree_nodes [DEBUG_TREE_LIMIT];
#endif /* DEBUG */

/* Fonctions externes du module : */
typedef struct {

  void (*close_debug_file) (void);
  void (*dump) (void);
  void (*open_debug_file) (void);
  int (*print) (const char *format, ...);
  void (*print_informations) (void);
  void (*print_statistics) (void);
  void (*split_debug_file) (void);
  void (*test_hash) (void);
  void (*trace) (char function[], bool is_printed, int *number_of_calls);

} debug_module_s;

/*******************************************************************************
  Data :
*******************************************************************************/

extern debug_module_s debug;

#ifdef DEBUG
extern FILE *debug_eval_file;
#endif /* DEBUG */

#ifdef DEBUG
extern FILE *debug_file;
#endif /* DEBUG */

#ifdef DEBUG
extern FILE *debug_tree_file;
#endif /* DEBUG */

#ifdef DEBUG
/* Du module : book */
extern int select_move_calls;
extern int make_book_calls;

/* Du module : evaluate */
extern int check_game_phase_calls;
extern int evaluate_position_calls;

/* Du module : hash */
extern int compute_h_position_calls;
extern int init_h_values_calls;
extern int update_h_position_calls;

/* Du module : main */
extern int call_uci_function_calls;
extern int exec_cmd_calls;
extern int exec_dbg_cmd_calls;
extern int init_game_calls;
extern int init_program_calls;
extern int main_calls;
extern int start_thinking_calls;

/* Du module : moves */
extern int generate_calls;
extern int is_attacked_calls;
extern int is_legal_calls;
extern int make_calls;
extern int unmake_calls;

/* Du module : search */
extern int apply_heuristic_calls;
extern int pick_move_calls;
extern int quiescence_search_calls;
extern int search_calls;
extern int sort_moves_calls;
extern int think_calls;
extern int update_killers_calls;

/* Du module : tools */
extern int compute_time_calls;
extern int convert_lan_move_calls;
extern int convert_pgm_move_calls;
extern int generate_random_calls;
extern int is_repeated_calls;
extern int print_board_calls;
extern int read_input_calls;
extern int send_output_calls;
extern int update_pieces_calls;
extern int verify_move_calls;

/* Du module : uci */
extern int go_calls;
extern int info_calls;
extern int isready_calls;
extern int position_calls;
extern int quit_calls;
extern int uci_cmd_calls;
extern int ucinewgame_calls;

/* Autres variables : */
extern bool debug_evaluation;
extern bool debug_tree;
extern int debug_file_indice;
extern int debug_node;
#endif /* DEBUG */

#endif /* DEBUG_H */
