/*******************************************************************************
  File : main.h

  Description : Fichier en-tte du module principal.
*******************************************************************************/

#ifndef MAIN_H
#define MAIN_H

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte de la librairie standard du langage C : */
#include <ctype.h>
#include <limits.h>
#include <signal.h>
#include <stdarg.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/*******************************************************************************
  Constants :
*******************************************************************************/

/* Informations sur le programme */
#define PRODUCT_AUTHOR       "Christophe J. Mandin"
#define PRODUCT_NAME	     "Sissa"
#define PRODUCT_VERSION      "1.0.0"

/* Nom des fichiers utiliss par le programme : */
#define CONFIG_FILE          "etc/config_file.txt"
#define GAMES_FILE           "etc/games.txt"
#define HELP_FILE            "etc/help_file.txt"
#define LOG_FILE             "log/log_file.txt"

/* Valeurs limites pour quelques constantes gnrales : */
#define BOOK_PLY_LIMIT           20
#define MIN_MAX_LIMIT       1000000
#define MOVES_LIMIT             256
#define PV_LIMIT               1024
#define STRING_LENGTH          4096

/* Noms des huit colonnes de l'chiquier : */
#define A 1
#define B 2
#define C 3
#define D 4
#define E 5
#define F 6
#define G 7
#define H 8

/* Noms des couleurs des deux camps : */
#define BLACK   0
#define WHITE   1

/* Valeurs du contenu des cases de l'chiquier : */
#define NONE     0

#define BPAWN    1
#define BKNIGHT  2
#define BBISHOP  3
#define BROOK    4
#define BQUEEN   5
#define BKING    6

#define WPAWN    9
#define WKNIGHT 10
#define WBISHOP 11
#define WROOK   12
#define WQUEEN  13
#define WKING   14

#define FRAME   15

/* Noms pour quelques coups spciaux : */
#define KING_CASTLING  1
#define QUEEN_CASTLING 2
#define EN_PASSANT     3
#define PROMOTION      4

/* Noms des diffrentes phases de la partie : */
#define OPENING      0
#define MIDDLE       1
#define ENDGAME      2
#define UNDEFINED_GS 3

/* Noms des rsultats possibles pour une partie : */
#define NO_RESULT          0
#define BLACK_IS_MATE      1
#define WHITE_IS_MATE      2
#define DRAW_FIFTY_MOVES   3
#define DRAW_REPETITION    4
#define DRAW_STALEMATE     5

/*******************************************************************************
  Types :
*******************************************************************************/

/* Fonctions externes du module : */
typedef struct {

  void (*initialize_game) (const char string[]);
  void (*start_thinking) (void);

} main_module_s;

/*******************************************************************************
  Data :
*******************************************************************************/

extern main_module_s main_module;

#endif /* MAIN_H */
