/*******************************************************************************
  File : tools.c

  Description : Diverses fonctions utilises tout au long du programme.
*******************************************************************************/

/*******************************************************************************
  Include files :
*******************************************************************************/

/* Fichiers en-tte des diffrents modules du programme : */
#include "debug.h"
#include "evaluate.h"
#include "hash.h"
#include "moves.h"
#include "tools.h"
#include "uci.h"

/*******************************************************************************
  External functions :
*******************************************************************************/

/*******************************************************************************
  Function     : compute_time
  Description  : Avant chaque coup du programme, cette fonction calcule le temps
                 octroy au programme avant d'arrter sa rflexion.
                 Les temps sont calculs en millimes de secondes.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void compute_time (void) {

/* ------------------------------- Local data ------------------------------- */

bool time_is_minimum = false; /* Indique si la limite infrieure est atteinte. */
bool time_is_maximum = false; /* Indique si la limite suprieure est atteinte. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("compute_time", true, &compute_time_calls);
#endif /* DEBUG */

  /* En mode UCI, le programme calcule un temps par coup en fonction du temps
     allou pour la partie : */
  if (uci_mode) {

    /* Pour la premire fois, */
    if (base_time == 0) {

      /* le temps de base allou est arbitrairement calcul sur une base de 60
         coups pour toute la partie. */
      base_time = (computer_color ? uci_wtime : uci_btime) / 60.0;
      time_for_move = base_time;
    }

    /* Si le programme est en avance au temps par rapport  son adversaire, */
    if ((computer_color ? uci_wtime : uci_btime) >
        (computer_color ? uci_btime : uci_wtime)) {

      /* et si l'avance est suffisamment importante */
      if (abs(uci_wtime - uci_btime) > base_time) {

        /* alors le temps allou pour le prochain coup est augment de 10%, */
        time_for_move = time_for_move * 1.1;

          /* mais pour viter un emballement, une limite maximum est fixe. */
        if (time_for_move > 2 * base_time) {

          time_for_move = 2 * base_time;
          time_is_maximum = true;
        }
      }
    }
    else {

      /* sinon le temps allou est diminu de 20%, */
        time_for_move = time_for_move * 0.8;

       /* mais pour permettre une certaine pertinence de l'analyse, une limite
          minimum est fixe. */
       if (time_for_move < base_time / 2.5) {

         time_for_move = base_time / 2.5;
         time_is_minimum = true;
       }
    }

    /* S'il reste moins d'une minure, */
    if ((computer_color ? uci_wtime : uci_btime) < 60000) {

       /* un temps maximum est octroy pour viter le zeitnot (la profondeur
          minimum de rflexion vitera les cas limites) : */
       if (time_for_move > (computer_color ? uci_wtime : uci_btime) / 10.0) {

         time_for_move = (computer_color ? uci_wtime : uci_btime) / 10.0;
         time_is_maximum = true;
       }
    }

    if (!test_mode) {

      tools.send_output (true, "info string Time for move : %.1f s (base time "
        "%.1f%s)\n", time_for_move / 1000.0, base_time / 1000.0,
        time_is_maximum ? " - time is limited" :
        time_is_minimum ? " - minimum time allowed" : "");
    }
  }
  /* En mode console, */
  else {

    /* lorsque aucun temps n'est dfinit pour un coup, */
    if (search_on_depth_limit) {

      /* l'arrt de la rflexion se fait en premier sur la profondeur de
         recherche, mais une limite est fixe. */
      time_for_move = maximum_search_time;
    }
    else {

      /* sinon le temps de rflexion est le mme pour chaque coup. */
      time_for_move = time_in_console_mode;
    }
  }

#ifdef DEBUG
debug.print (" : time_for_move %d\n", time_for_move);
#endif /* DEBUG */
}
/* End of function : compute_time */

/*******************************************************************************
  Function     : convert_lan_move
  Description  : Pour convertir un coup du format LAN (Long Agebraic Notation),
                 vers le format tel qu'il est connu par le programme (voir la
                 description de la structure "move_s").
  Parameters   : in  - coup au format LAN
                 out - coup au format du programme
  Return value : Le coup est potentiellement correct (true) ou non (false). Cela
                 ne signifie cependant pas qu'il fait partie des coups jouables,
                 ni qu'il est lgal.
  Validation   : 1
*******************************************************************************/
bool convert_lan_move (const char *lan_move, move_s *pgm_move) {

/* ------------------------------- Local data ------------------------------- */

bool is_possible_move = false;    /* Par dfaut le coup n'est pas correct. */

int x, y;     /* Colonne et range d'une case de l'chiquier. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("convert_lan_move\n", false, &convert_lan_move_calls);
#endif /* DEBUG */

  /* Initialisation des informations lies au coup. */
  memset (pgm_move, 0, sizeof (move_s));

  /* Calcul des cases de dpart et d'arrive. */
  if (*lan_move >= 'a' && *lan_move <= 'h') {

    x = (*lan_move) - 'a' + 1;

    lan_move++;
    if (*lan_move >= '1' && *lan_move <= '8') {

      y = (*lan_move) - '1' + 1;
      pgm_move->from = (y + 1) * 10 + x;

      lan_move++;
      if (*lan_move >= 'a' && *lan_move <= 'h') {

        x = (*lan_move) - 'a' + 1;

        lan_move++;
        if (*lan_move >= '1' && *lan_move <= '8') {

          y = (*lan_move) - '1' + 1;
          pgm_move->to = (y + 1) * 10 + x;

          /* Le coup est au format LAN. */
          is_possible_move = true;
        }
      }
    }
  }

  /* Recherche d'une ventuelle promotion de pion. */
  lan_move++;
  switch (*lan_move) {

    case 'n' :     /* Cavalier */

      pgm_move->status = PROMOTION;
      pgm_move->promoted = (active_color ? WKNIGHT : BKNIGHT);
      break;

    case 'b' :     /* Fou */

      pgm_move->status = PROMOTION;
      pgm_move->promoted = (active_color ? WBISHOP : BBISHOP);
      break;

    case 'r' :     /* Tour */

      pgm_move->status = PROMOTION;
      pgm_move->promoted = (active_color ? WROOK : BROOK);
      break;

    case 'q' :     /* Dame */

      pgm_move->status = PROMOTION;
      pgm_move->promoted = (active_color ? WQUEEN : BQUEEN);
      break;
  }

  /* Vrifie la prsence d'une pice sur la case de dpart. */
  if (board[pgm_move->from]) {

    /* Vrifie si le coup permet la capture d'une pice. */
    if (board[pgm_move->to]) {

      pgm_move->captured = board[pgm_move->to];
      pgm_move->captured_ptr = pieces_ptr[pgm_move->to];
    }

    /* Vrifie si le coup est une prise en passant. */
    if (   (board[pgm_move->from] & 0x7) == BPAWN
        && !board[pgm_move->to]
        && file[pgm_move->from] != file[pgm_move->to]) {

      pgm_move->status = EN_PASSANT;
      pgm_move->captured = (active_color ? BPAWN : WPAWN);
      pgm_move->captured_ptr =
        pieces_ptr[(rank[pgm_move->from] + 1) * 10 + file[pgm_move->to]];
    }

    /* Vrifie si le coup est un roque. */
    if (   (board[pgm_move->from] & 0x7) == BKING
        && abs(pgm_move->to - pgm_move->from) == 2) {

      if (file[pgm_move->to] == G) {

        pgm_move->status = KING_CASTLING;
      }
      else if (file[pgm_move->to] == C) {

        pgm_move->status = QUEEN_CASTLING;
      }
    }
  }
  else {

    is_possible_move = false;
  }

  return is_possible_move;
}
/* End of function : convert_lan_move */

/*******************************************************************************
  Function     : convert_pgm_move
  Description  : Pour convertir un coup du format tel qu'il est connu par le
                 programme, vers le format LAN (Long Algebraic Notation).
  Parameters   : in  - coup au format du programme
                 out - coup au format LAN
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void convert_pgm_move (move_s *pgm_move, char lan_move[]) {

/* ------------------------------- Local data ------------------------------- */

char promoted_char;     /* Pour mmoriser la pice de promotion. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("convert_pgm_move\n", false, &convert_pgm_move_calls);
#endif /* DEBUG */

  /* Dans le cas d'une promotion, traduction de la pice promue : */
  switch (pgm_move->promoted) {
    case (BKNIGHT) : case (WKNIGHT) : promoted_char = 'n'; break;
    case (BBISHOP) : case (WBISHOP) : promoted_char = 'b'; break;
    case (BROOK)   : case (WROOK)   : promoted_char = 'r'; break;
    case (BQUEEN)  : case (WQUEEN)  : promoted_char = 'q'; break;
    default :
      promoted_char = '\0';
  }

  /* Traduction du coup complet (case de dpart + case d'arrive + promotion
     ventuelle) : */
  sprintf (lan_move, "%c%d%c%d%c",
           file[pgm_move->from] + (int) 'a' - 1, rank[pgm_move->from],
           file[pgm_move->to] + (int) 'a' - 1, rank[pgm_move->to],
           promoted_char);
}
/* End of function : convert_pgm_move */

/*******************************************************************************
  Function     : generate_random
  Description  : Gnration alatoire d'un nombre entier non sign sur 32 bits.
  Parameters   : aucun
  Return value : nombre alatoire
  Validation   : 1
*******************************************************************************/
unsigned int generate_random (void) {

/* ------------------------------- Local data ------------------------------- */

int byte1, byte2, byte3, byte4;
unsigned int random_number;

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("generate_random\n", false, &generate_random_calls);
#endif /* DEBUG */

  /* Gnration de 4 nombres compris dans l'intervalle [0;256[ avec une
     prcision de 1/RAND_MAX */
  byte1 = (int) ((double) rand() * 8 / (RAND_MAX + 1));
  byte2 = (int) ((double) rand() * 8 / (RAND_MAX + 1));
  byte3 = (int) ((double) rand() * 8 / (RAND_MAX + 1));
  byte4 = (int) ((double) rand() * 8 / (RAND_MAX + 1));

  /* Concatnation des 4 octets pour former un nombre de 32 bits : */
  random_number = byte1 + (byte2 << 8) + (byte3 << 16) + (byte4 << 24);

  return random_number;
}
/* End of function : generate_random */

/*******************************************************************************
  Function     : is_repeated
  Description  : Recherche si la position courante se retrouve pour la troisime
                 fois sur l'chiquier depuis le dbut de la partie.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
bool is_repeated (void) {

/* ------------------------------- Local data ------------------------------- */

int count = 1;     /* Compte les positions identiques  la position courante. */
int i;             /* Indice de boucle. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("is_repeated", true, &is_repeated_calls);
#endif /* DEBUG */

  /* La recherche n'a de sens que si au moins 6 demi-coups ont t jous. */
  if (halfmove >= 6) {

    /* La recherche se poursuit tous les 2 demi-coups jusqu'au dernier
       dplacement de pion ou la dernire capture. */
    for (i = game_ply - 2; i >= game_ply - halfmove; i -= 2) {

      /* La comparaison des positions se fait avec leur valeur de hachage. */
      if (   h_position.h1 == h_position_history[i].h1
          && h_position.h2 == h_position_history[i].h2) {

        count++;
      }

      if (count > 2) {

#ifdef DEBUG
debug.print (" : true\n");
#endif /* DEBUG */
        return true;
      }
    }
  }

  /* Mmorise le nombre de rptitions de la position, pour en tenir compte
     dans le module d'valuation (peut prendre les valeurs 1 ou 2). */
  repeat_counter = count;

#ifdef DEBUG
debug.print (" : false (count %d)\n", count);
#endif /* DEBUG */
  return false;
}
/* End of function : is_repeated */

/*******************************************************************************
  Function     : print_board
  Description  : Envoie une reprsentation de l'chiquier vers une destination
                 donne en entre (console ou autre fichier).
  Parameters   : in  - destination de l'impression de l'chiquier
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void print_board (FILE *output_file) {

/* ------------------------------- Local data ------------------------------- */

char *line = "   |---+---+---+---+---+---+---+---|";
                        /* Ligne de sparation de l'chiquier. */

char *piece_char[15] = {
  " ", "p", "n", "b", "r", "q", "k", " ", " ", "P", "N", "B", "R", "Q", "K" };
                        /* Caractres de reprsentation des pices du jeu. */

int square;             /* Case de l'chiquier. */
int x, y;               /* Colonne et range de l'chiquier. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("print_board\n", false, &print_board_calls);
#endif /* DEBUG */

  fprintf (output_file, "\n  %s\n", line);

  for (y = 8; y > 0; y--) {

    fprintf (output_file, "   %d |", y);

    for (x = 1; x < 9; x++) {

      square = (y + 1) * 10 + x;
      fprintf (output_file, " %s |", piece_char[board[square]]);
    }

    if (test_mode) {

      if (y == 7) {

        fprintf (output_file, " %.2f (static evaluation)",
                (float) evaluate.evaluate_position () / 100);
      }

      if (y == 6) {

        if  (tools.is_repeated ()) {

          fprintf (output_file, " Game is draw");
        }
        else {

          fprintf (output_file, " %d (halfmove)", halfmove);
        }
      }

      if (y == 4) {

        fprintf (output_file, " %8x, %8x (hash value)",
          h_position.h1, h_position.h2);
      }

      if (y == 3) {

        fprintf (output_file, " %x (castle_status)", CASTLE_STATUS);
      }
    }

    if (y == 2) {

      fprintf (output_file, " %d (game_ply)", game_ply);
    }

    if (y == 1) {

      fprintf (output_file, active_color ? " WHITE to move" : " BLACK to move");
    }

    fprintf (output_file, "\n  %s\n", line);
  }

  fprintf (output_file, "       a   b   c   d   e   f   g   h\n");
}
/* End of function : print_board */

/*******************************************************************************
  Function     : read_input
  Description  : Lecture de caractres  partir de l'entre standard.
  Parameters   : out - chane des caractres lus
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void read_input (char string[]) {

/* ------------------------------- Local data ------------------------------- */

int character;               /* Caractre lu sur l'entre standard. */
int i = 0;                   /* Indice du caractre lu. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
  debug.trace ("read_input", true, &read_input_calls);
#endif /* DEBUG */

  /* Lecture d'une suite de caractres. */
  while ((character = getc (stdin)) != (int) '\n' && character != EOF) {

    if (i < STRING_LENGTH - 1) {

      /* Construction de la chane. */
      string[i++] = character;
    }
  }

  /* Terminaison de la chane. */
  string[i] = '\0';

  if (log_use) {

    /* Pour tracer la saisie dans le fichier journal : */
    fprintf (log_file, "\nInput  : %s\n", string);
  }

#ifdef DEBUG
  debug.print (" : %s\n", string);
#endif /* DEBUG */
}
/* End of function : read_input */

/*******************************************************************************
  Function     : send_output
  Description  : Pour envoyer des informations vers les diffrentes sorties
                 (console, fichier journal, fichier de dbogage).
  Parameters   : in  - indique si les donnes sont  envoyer sur la console
                 in  - format d'impression des donnes
                 in  - liste de longueur variable des donnes  envoyer
  Return value : nombre de caractres crits
  Validation   : 1
*******************************************************************************/
int send_output (bool to_stdout, const char *format, ...) {

/* ------------------------------- Local data ------------------------------- */

int size = 0;                /* Nombre de caractres crits. */
va_list list;                /* Liste variable des arguments. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
  debug.trace ("send_output", true, &send_output_calls);
#endif /* DEBUG */

  /* Initialisation de la liste des paramtres. */
  va_start (list, format);

  if (to_stdout) {

    /* Envoi des donnes vers la console. */
    size = vfprintf (stdout, format, list);
  }

  /* Si c'est autoris, */
  if (log_use) {

    /* Les donnes sont crites dans le fichier journal. */
    size = vfprintf (log_file, format, list);
    fflush (log_file);
  }

  /* Fin de lecture de la liste des paramtres. */
  va_end (list);

  return size;
}
/* End of function : send_output */

/*******************************************************************************
  Function     : update_pieces
  Description  : Met  jour la table des pices et le lien entre la fiche de la
                 pice et la case de l'chiquier sur laquelle elle se trouve.
                 L'intrt de cette table est de tenir compte de la diminution
                 du nombre des pices au cours de la partie.
  Parameters   : aucun
  Return value : aucune
  Validation   : 1
*******************************************************************************/
void update_pieces (void) {

/* ------------------------------- Local data ------------------------------- */

int color;         /* Couleur du camp. */
int square;        /* Case de l'chiquier. */
int x, y;          /* Colonne et range de l'chiquier. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("update_pieces", true, &update_pieces_calls);
#endif /* DEBUG */

  /* Initialisation du nombre des pices blanches et noires. */
  pieces_number[BLACK] = 0;
  pieces_number[WHITE] = 0;

  for (y = 8; y >= 1; y--) {

    for (x = 1; x <= 8; x++) {

      square = (y + 1) * 10 + x;

      /* Si la case n'est pas vide, */
      if (board[square]) {

        /* alors calcule la couleur de la pice, */
        color = board[square] >> 3;

        /* comptabilise la pice trouve, */
        pieces_number[color]++;

        /* indique sur quel case se trouve chaque pice, */
        pieces[color][pieces_number[color]] = square;

        /* et tablit un lien direct entre la case et la pice. */
        pieces_ptr[square] = &pieces[color][pieces_number[color]];
      }
      else {

        /* Pour prendre en compte le fait que la position de dpart n'est pas
           toujours la position classique avec toutes les pices : */
        moved[square] = true;
      }
    }
  }

#ifdef DEBUG
debug.print (" : ");
for (color = BLACK; color <= WHITE; color++) {
  int i;
  for (i = 1; i <= pieces_number[color]; i++) {
    if (pieces[color][i]) {
      debug.print ("%c%c%d - ",
        PIECE(board[pieces[color][i]]),
        file[pieces[color][i]] + (int) 'a' -1,
        rank[pieces[color][i]]);
    }
  }
}
debug.print ("\n");
#endif /* DEBUG */
}
/* End of function : update_pieces */

/*******************************************************************************
  Function     : verify_move
  Description  : Pour vrifier si un coup au format LAN est lgal dans la
                 position courante.
  Parameters   : in  - coup  vrifier
  Return value : Le coup est lgal (true) ou non (false).
  Validation   : 1
*******************************************************************************/
bool verify_move (char input_lan_move[]) {

/* ------------------------------- Local data ------------------------------- */

bool legal = false;               /* Valeur de retour de la fonction. */

char lan_move[6];                 /* Coup au format LAN. */

int moves_number;                 /* Nombre de coups jouables. */

move_s moves_list[MOVES_LIMIT];   /* Pour ranger les coups d'une position. */

/* ------------------------------ Function code ----------------------------- */

#ifdef DEBUG
debug.trace ("verify_move\n", false, &verify_move_calls);
#endif /* DEBUG */

  /* Recherche la liste des coups jouables dans la position courante. */
  moves_number = moves.generate (moves_list, false);

  /* Parcourt la liste obtenue. */
  do {

    /* Conversion du coup pour permettre la comparaison. */
    tools.convert_pgm_move (&moves_list[moves_number], lan_move);

    /* Si le coup entr correspond  un coup jouable, */
    if (!strcmp (input_lan_move, lan_move)) {

      /* alors il est jou sur l'chiquier, */
      moves.make (&moves_list[moves_number]);

      /* pour vrifier sa lgalit. */
      if (moves.is_legal (&moves_list[moves_number])) {

        legal = true;
      }

      /* La position d'avant le test est remise en place. */
      moves.unmake (&moves_list[moves_number]);
    }

  } while (!legal && --moves_number);

  return legal;
}
/* End of function : verify_move */

/*******************************************************************************
  Object declaration :
*******************************************************************************/

tools_module_s tools = {

  compute_time,
  convert_lan_move,
  convert_pgm_move,
  generate_random,
  is_repeated,
  print_board,
  read_input,
  send_output,
  update_pieces,
  verify_move
};
